﻿/*
	Antenna House PDF Tool API V7.0
	.NET Interface sample program

	概要：セキュリティ情報の取得

	Copyright 2013-2021 Antenna House, Inc.
*/

using System;
using PdfTkNet;

namespace GetEncryptInfo
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length < 1)
            {
                Console.WriteLine("usage: GetEncryptInfo.exe in-pdf-file [in-pdf-password] [in-pfx]\n");
                return;
            }

            try
            {
                if (args.Length > 2)
                {
                    showEncryptPubKey(args);
                }
                else
                {
                    showEncryptStandard(args);
                }
            }
            catch (PtlException pex)
            {
                Console.WriteLine(pex.getErrorCode() + " : " + pex.getErrorMessageJP());
                pex.Dispose();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
            finally
            {
                Console.WriteLine("-- 完了 --");
            }
        }

        static void showEncryptStandard(string[] args)
        {
            using (PtlParamInput inputFile = new PtlParamInput(args[0]))
            using (PtlPDFDocument doc = new PtlPDFDocument())
            {
                if (args.Length > 1)
                {
                    // パスワードのセット
                    doc.setPassword(args[1]);
                }

                // PDFファイルをロードします。
                doc.load(inputFile);

                // 暗号化の取得
                if (doc.isEncrypted())
                {
                    // 暗号化情報の取得
                    showEncrypt(doc);
                }
                else
                {
                    Console.WriteLine("暗号化されていないファイルです");
                }
            }
        }

        static void showEncryptPubKey(string[] args)
        {
            using (PtlParamInput inputFile = new PtlParamInput(args[0]))
            using (PtlParamInput inputPfx = new PtlParamInput(args[2]))
            using (PtlPDFDocument doc = new PtlPDFDocument())
            {
                string password = (string)args[1];
                // パスワードのセット
                doc.setPassword(password);

                // PKCS12セット
                doc.setPKCS12(inputPfx);

                // PDFファイルをロードします。
                doc.load(inputFile);

                // 暗号化の取得
                if (doc.isEncrypted())
                {
                    // 暗号化情報の取得
                    showEncrypt(doc);
                }
                else
                {
                    Console.WriteLine("暗号化されていないファイルです");
                }
            }
        }

        static void showEncrypt(PtlPDFDocument doc)
        {
            // 暗号化情報の取得
            using (PtlEncrypt encrypt = doc.getEncrypt())	// 暗号化されていないと例外が発生
            {
                // キー長の取得
                Console.WriteLine("KeyLength : " + encrypt.getKeyLength());

                // 暗号化する文書コンポーネント取得
                switch (encrypt.getEncryptComponent())
                {
                    case PtlEncrypt.ENCRYPT_COMPONENT.ENCRYPT_ALL:
                        Console.WriteLine("文書の全てのコンテンツを暗号化");
                        break;
                    case PtlEncrypt.ENCRYPT_COMPONENT.ENCRYPT_EXCEPT_METADATA:
                        Console.WriteLine("文書のメタデータを除く全てのコンテンツを暗号化");
                        break;
                    case PtlEncrypt.ENCRYPT_COMPONENT.ENCRYPT_ONLY_ATTACHED_FILE:
                        Console.WriteLine("添付ファイルのみを暗号化(AES暗号化のみで有効)");
                        break;
                }

                // 暗号化のフィルター取得
                switch (encrypt.getFilterType())
                {
                    case PtlEncrypt.FILTER_TYPE.FILTER_STANDARD:
                        PtlEncryptStandard encryptStandard = (PtlEncryptStandard)encrypt;

                        // 標準セキュリティハンドラのメソッド取得
                        switch (encryptStandard.getMethod())
                        {
                            case PtlEncryptStandard.STANDARD_METHOD_TYPE.METHOD_RC4:
                                Console.WriteLine("標準セキュリティハンドラのメソッド : RC4");
                                break;
                            case PtlEncryptStandard.STANDARD_METHOD_TYPE.METHOD_AES:
                                Console.WriteLine("標準セキュリティハンドラのメソッド : AES");
                                break;
                        }

                        // ユーザアクセス許可フラグの取得
                        using (PtlEncryptPermission perms = encryptStandard.getPermission())
                        {
                            // タイプ取得
                            switch (perms.getType())
                            {
                                case PtlEncryptPermission.PERM_TYPE.PERM_TYPE1:
                                    Console.WriteLine("RC4 40 ビット 暗号化での権限");
                                    showPermissionType1(perms);
                                    break;
                                case PtlEncryptPermission.PERM_TYPE.PERM_TYPE2:
                                    Console.WriteLine("RC4 40 ビット以外の暗号化での権限");
                                    showPermissionType2(perms);
                                    break;
                            }
                        }
                        break;
                    case PtlEncrypt.FILTER_TYPE.FILTER_PUBKEY:
                        PtlEncryptPubKey encryptPubKey = (PtlEncryptPubKey)encrypt;

                        // 公開キーセキュリティハンドラのメソッド取得
                        switch (encryptPubKey.getMethod())
                        {
                            case PtlEncryptPubKey.PUBSEC_METHOD_TYPE.METHOD_RC4:
                                Console.WriteLine("公開キーセキュリティハンドラのメソッド : RC4");
                                break;
                            case PtlEncryptPubKey.PUBSEC_METHOD_TYPE.METHOD_AES:
                                Console.WriteLine("公開キーセキュリティハンドラのメソッド : AES");
                                break;
                        }

                        // ユーザアクセス許可フラグの取得
                        using (PtlEncryptPermission perms = encryptPubKey.getPermission())
                        {
                            PtlEncryptPermissionPubKey permsPubKey = (PtlEncryptPermissionPubKey)perms;
                            bool fullPermission = permsPubKey.hasFullPermission();
                            if (fullPermission)
                            {
                                Console.WriteLine("全ての権限がある\n");
                                Console.WriteLine("以下の権限情報は無視される\n");
                            }
                            else
                            {
                                Console.WriteLine("全ての権限がある訳ではない\n");
                                Console.WriteLine("以下の権限情報に従う\n");
                            }

                            showPermissionType2(perms);
                        }
                        break;
                }
            }
        }

        static void showPermissionType1(PtlEncryptPermission perms)
        {
            PtlEncryptPermissionType1 perms1 = (PtlEncryptPermissionType1)perms;
            //印刷権限取得
            if (perms1.getPrint())
            {
                Console.WriteLine("印刷を許可する");
            }
            else
            {
                Console.WriteLine("印刷を許可しない");
            }
            //変更権限取得
            switch (perms1.getModify())
            {
                case PtlEncryptPermissionType1.PERMISSION_MODIFY.PERM_MODIFY_NOT_ALLOWED:
                    Console.WriteLine("変更権限 : 許可しない");
                    break;
                case PtlEncryptPermissionType1.PERMISSION_MODIFY.PERM_MODIFY_ANNOTANDFORM:
                    Console.WriteLine("変更権限 : 注釈の作成、フォームフィールドの入力と既存の署名フィールドに署名");
                    break;
                case PtlEncryptPermissionType1.PERMISSION_MODIFY.PERM_MODIFY_ASSEMBLEANDFORM:
                    Console.WriteLine("変更権限 : ページレイアウト、フォームフィールドの入力と既存の署名フィールドに署名");
                    break;
                case PtlEncryptPermissionType1.PERMISSION_MODIFY.PERM_MODIFY_MODYFYDOC:
                    Console.WriteLine("変更権限 : ページ抽出を除く全ての動作");
                    break;
            }
            //テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にするかどうかの取得
            if (perms1.getCopy())
            {
                Console.WriteLine("テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にする");
            }
            else
            {
                Console.WriteLine("テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にしない");
            }
        }
        static void showPermissionType2(PtlEncryptPermission perms)
        {
            PtlEncryptPermissionType2 perms2 = (PtlEncryptPermissionType2)perms;
            //印刷権限取得
            switch (perms2.getPrint())
            {
                case PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_NOT_ALLOWED:
                    Console.WriteLine("印刷権限 : 許可しない");
                    break;
                case PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_LOW:
                    Console.WriteLine("印刷権限 : 低解像度");
                    break;
                case PtlEncryptPermissionType2.PERMISSION_PRINT.PERM_PRINT_HIGH:
                    Console.WriteLine("印刷権限 : 高解像度");
                    break;
            }
            //変更権限取得
            switch (perms2.getModify())
            {
                case PtlEncryptPermissionType2.PERMISSION_MODIFY.PERM_MODIFY_NOT_ALLOWED:
                    Console.WriteLine("変更権限 : 許可しない");
                    break;
                case PtlEncryptPermissionType2.PERMISSION_MODIFY.PERM_MODIFY_ASSEMBLEDOC:
                    Console.WriteLine("変更権限 : ページの挿入、削除、回転");
                    break;
                case PtlEncryptPermissionType2.PERMISSION_MODIFY.PERM_MODIFY_FILLFORM:
                    Console.WriteLine("変更権限 : フォームフィールドの入力と既存の署名フィールドに署名");
                    break;
                case PtlEncryptPermissionType2.PERMISSION_MODIFY.PERM_MODIFY_ANNOTANDFORM:
                    Console.WriteLine("変更権限 : 注釈の作成、フォームフィールドの入力と既存の署名フィールドに署名");
                    break;
                case PtlEncryptPermissionType2.PERMISSION_MODIFY.PERM_MODIFY_MODYFYDOC:
                    Console.WriteLine("変更権限 : ページ抽出を除く全ての動作");
                    break;
            }
            //テキスト、画像、その他の内容のコピーを有効にするかどうかの取得
            if (perms2.getCopy())
            {
                Console.WriteLine("テキスト、画像、その他の内容のコピーを有効にする");
            }
            else
            {
                Console.WriteLine("テキスト、画像、その他の内容のコピーを有効にしない");
            }
            //スクリーンリーダーデバイスのテキストアクセスを有効にするかどうかの取得
            if (perms2.getAccessibility())
            {
                Console.WriteLine("スクリーンリーダーデバイスのテキストアクセスを有効にする");
            }
            else
            {
                Console.WriteLine("スクリーンリーダーデバイスのテキストアクセスを有効にしない");
            }
        }
    }
}
