/*
    Antenna House PDF Tool API V7.0
    Java Interface sample program

    概要：セキュリティ情報の取得

    Copyright 2015-2021 Antenna House, Inc.
*/

package Sample;

import java.io.*;
import jp.co.antenna.ptl.*;

public class GetEncryptInfo {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length < 1)
        {
            System.out.println("usage: java GetEncryptInfo in-pdf-file out-xfdf-file [in-pdf-password] [in-pfx]");
            return;
        }

        try
        {
            if (args.length > 2)
            {
                showEncryptPubKey(args);
            }
            else
            {
                showEncryptStandard(args);
            }
        }
        catch (PtlException pex) {
            System.out.println("PtlException : ErrorCode = " + pex.getErrorCode() + "\n  " + pex.getErrorMessage());
        }
        catch (Exception ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        catch (Error ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        finally {
            System.out.println("-- 完了 --");
        }
    }

    static void showEncryptStandard(String[] args) throws PtlException
    {
        try (PtlParamInput inputFile = new PtlParamInput(args[0]);
             PtlPDFDocument doc = new PtlPDFDocument())
        {
            if (args.length > 1)
            {
                // パスワードのセット
                doc.setPassword(args[1]);
            }

            // PDFファイルをロードします。
            doc.load(inputFile);

            // 暗号化の取得
            if (doc.isEncrypted())
            {
                // 暗号化情報の取得
                showEncrypt(doc);
            }
            else
            {
                System.out.println("暗号化されていないファイルです");
            }
        }
    }

    static void showEncryptPubKey(String[] args) throws PtlException
    {
        try (PtlParamInput inputFile = new PtlParamInput(args[0]);
             PtlParamInput inputPfx = new PtlParamInput(args[2]);
             PtlPDFDocument doc = new PtlPDFDocument())
        {
            // パスワードのセット
            doc.setPassword(args[1]);

            // PKCS12セット
            doc.setPKCS12(inputPfx);

            // PDFファイルをロードします。
            doc.load(inputFile);

            // 暗号化の取得
            if (doc.isEncrypted())
            {
                // 暗号化情報の取得
                showEncrypt(doc);
            }
            else
            {
                System.out.println("暗号化されていないファイルです");
            }
        }
    }

    static void showEncrypt(PtlPDFDocument doc) throws PtlException
    {
        // 暗号化情報の取得
        try (PtlEncrypt encrypt = doc.getEncrypt())	// 暗号化されていないと例外が発生
        {
            // キー長の取得
            System.out.println("KeyLength : " + encrypt.getKeyLength());

            // 暗号化する文書コンポーネント取得
            switch (encrypt.getEncryptComponent())
            {
            case ENCRYPT_ALL:
                System.out.println("文書の全てのコンテンツを暗号化");
                break;
            case ENCRYPT_EXCEPT_METADATA:
                System.out.println("文書のメタデータを除く全てのコンテンツを暗号化");
                break;
            case ENCRYPT_ONLY_ATTACHED_FILE:
                System.out.println("添付ファイルのみを暗号化(AES暗号化のみで有効)");
                break;
            }

            // 暗号化のフィルター取得
            switch (encrypt.getFilterType())
            {
            case FILTER_STANDARD:
                PtlEncryptStandard encryptStandard = (PtlEncryptStandard)encrypt;

                // 標準セキュリティハンドラのメソッド取得
                switch (encryptStandard.getMethod())
                {
                case METHOD_RC4:
                    System.out.println("標準セキュリティハンドラのメソッド : RC4");
                    break;
                case METHOD_AES:
                    System.out.println("標準セキュリティハンドラのメソッド : AES");
                    break;
                }

                // ユーザアクセス許可フラグの取得
                try (PtlEncryptPermission perms = encryptStandard.getPermission())
                {
                    // タイプ取得
                    switch (perms.getType())
                    {
                    case PERM_TYPE1:
                        System.out.println("RC4 40 ビット 暗号化での権限");
                        showPermissionType1(perms);
                        break;
                    case PERM_TYPE2:
                        System.out.println("RC4 40 ビット以外の暗号化での権限");
                        showPermissionType2(perms);
                        break;
                    }
                }
                break;
            case FILTER_PUBKEY:
                PtlEncryptPubKey encryptPubKey = (PtlEncryptPubKey)encrypt;

                // 公開キーセキュリティハンドラのメソッド取得
                switch (encryptPubKey.getMethod())
                {
                case METHOD_RC4:
                    System.out.println("公開キーセキュリティハンドラのメソッド : RC4");
                    break;
                case METHOD_AES:
                    System.out.println("公開キーセキュリティハンドラのメソッド : AES");
                    break;
                }

                // ユーザアクセス許可フラグの取得
                try (PtlEncryptPermission perms = encryptPubKey.getPermission())
                {
                    PtlEncryptPermissionPubKey permsPubKey = (PtlEncryptPermissionPubKey)perms;
                    boolean fullPermission = permsPubKey.hasFullPermission();
                    if (fullPermission)
                    {
                        System.out.println("全ての権限がある\n");
                        System.out.println("以下の権限情報は無視される\n");
                    }
                    else
                    {
                        System.out.println("全ての権限がある訳ではない\n");
                        System.out.println("以下の権限情報に従う\n");
                    }

                    showPermissionType2(perms);
                }
                break;
            }
        }
    }

    static void showPermissionType1(PtlEncryptPermission perms) throws PtlException
    {
        PtlEncryptPermissionType1 perms1 = (PtlEncryptPermissionType1)perms;
        //印刷権限取得
        if (perms1.getPrint())
        {
            System.out.println("印刷を許可する");
        }
        else
        {
            System.out.println("印刷を許可しない");
        }
        //変更権限取得
        switch (perms1.getModify())
        {
        case PERM_MODIFY_NOT_ALLOWED:
            System.out.println("変更権限 : 許可しない");
            break;
        case PERM_MODIFY_ANNOTANDFORM:
            System.out.println("変更権限 : 注釈の作成、フォームフィールドの入力と既存の署名フィールドに署名");
            break;
        case PERM_MODIFY_ASSEMBLEANDFORM:
            System.out.println("変更権限 : ページレイアウト、フォームフィールドの入力と既存の署名フィールドに署名");
            break;
        case PERM_MODIFY_MODYFYDOC:
            System.out.println("変更権限 : ページ抽出を除く全ての動作");
            break;
        }
        //テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にするかどうかの取得
        if (perms1.getCopy())
        {
            System.out.println("テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にする");
        }
        else
        {
            System.out.println("テキスト、画像、その他の内容のコピーおよびアクセシビリティを有効にしない");
        }
    }

    static void showPermissionType2(PtlEncryptPermission perms)	throws PtlException
    {
        PtlEncryptPermissionType2 perms2 = (PtlEncryptPermissionType2)perms;
        //印刷権限取得
        switch (perms2.getPrint())
        {
        case PERM_PRINT_NOT_ALLOWED:
            System.out.println("印刷権限 : 許可しない");
            break;
        case PERM_PRINT_LOW:
            System.out.println("印刷権限 : 低解像度");
            break;
        case PERM_PRINT_HIGH:
            System.out.println("印刷権限 : 高解像度");
            break;
        }
        //変更権限取得
        switch (perms2.getModify())
        {
        case PERM_MODIFY_NOT_ALLOWED:
            System.out.println("変更権限 : 許可しない");
            break;
        case PERM_MODIFY_ASSEMBLEDOC:
            System.out.println("変更権限 : ページの挿入、削除、回転");
            break;
        case PERM_MODIFY_FILLFORM:
            System.out.println("変更権限 : フォームフィールドの入力と既存の署名フィールドに署名");
            break;
        case PERM_MODIFY_ANNOTANDFORM:
            System.out.println("変更権限 : 注釈の作成、フォームフィールドの入力と既存の署名フィールドに署名");
            break;
        case PERM_MODIFY_MODYFYDOC:
            System.out.println("変更権限 : ページ抽出を除く全ての動作");
            break;
        }
        //テキスト、画像、その他の内容のコピーを有効にするかどうかの取得
        if (perms2.getCopy())
        {
            System.out.println("テキスト、画像、その他の内容のコピーを有効にする");
        }
        else
        {
            System.out.println("テキスト、画像、その他の内容のコピーを有効にしない");
        }
        //スクリーンリーダーデバイスのテキストアクセスを有効にするかどうかの取得
        if (perms2.getAccessibility())
        {
            System.out.println("スクリーンリーダーデバイスのテキストアクセスを有効にする");
        }
        else
        {
            System.out.println("スクリーンリーダーデバイスのテキストアクセスを有効にしない");
        }
    }
}

