/*
	Antenna House PDF Tool API V7.0
	.NET Interface sample program

	概要：画像の入れ替え

	Copyright 2021 Antenna House,Inc.
*/

package Sample;

import jp.co.antenna.ptl.*;

public class ReplaceImage {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length < 3)
        {
            System.out.println("usage: java ReplaceImage in-pdf-file out-pdf-file image-file");
            return;
        }

        try (PtlOption option = new PtlOption();
             PtlParamInput input = new PtlParamInput(args[0]);
             PtlParamOutput output = new PtlParamOutput(args[1]);
             PtlPDFDocument doc = new PtlPDFDocument())
        {
            option.setUnit(PtlOption.UNIT.UNIT_PT); // Matirixを扱うため

            // PDFファイルをロードします。
            doc.load(input);

            try (PtlPages pages = doc.getPages())
            {
                int pageCount = pages.getCount();
                for (int i = 0; i < pageCount; ++i)
                {
                    try (PtlPage page = pages.get(i);
                         PtlContent contentPage = page.getContent())
                    {
                        // ページコンテントをパースして画像を置き換える
                        if (replaceImage(contentPage, args[2]))
                        {
                            page.setContent(contentPage);   // ページコンテントを置き換える
                        }
                    }
                }
            }
            doc.save(output);
        }
        catch (PtlException pex) {
            System.out.println("PtlException : ErrorCode = " + pex.getErrorCode() + "\n  " + pex.getErrorMessage());
        }
        catch (Exception ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        catch (Error ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        finally {
            System.out.println("-- 完了 --");
        }
    }

    static boolean replaceImage(PtlContent content, String imageParh)
    {
        boolean ret = false;
        try
        {
            try (PtlEditElements elems = content.getEditElements())
            {
                if (elems.isEmpty()) return false;
                int elementcount = elems.getCount();
                for (int i = 0; i < elementcount; ++i)
                {
                    try (PtlEditElement elem = elems.get(i))
                    {
                        switch (elem.getType())
                        {
                        case TYPE_FORMXOBJECT:
                            {
                                PtlEditFormXObject elemForm = (PtlEditFormXObject)elem;
                                try (PtlContent contentForm = elemForm.getContent())  // FormXObjectコンテント取得
                                {
                                    // FormXObjectコンテントをパースして画像を置き換える
                                    if (replaceImage(contentForm, imageParh))
                                    {
                                        elemForm.setContent(contentForm);   // FormXObjectコンテントを置き換える
                                        // trueは返さない
                                        // 何故ならPtlEditFormXObjectでsetContentしたので上位でsetContentする必要がない
                                    }
                                }
                            }
                            break;
                        case TYPE_MCCONTAINER:
                            {
                                PtlEditMCContainer elemContainer = (PtlEditMCContainer)elem;
                                try (PtlContent contentContainer = elemContainer.getContent())    // マークコンテント取得
                                {
                                    // マークコンテントをパースして画像を置き換える
                                    if (replaceImage(contentContainer, imageParh))
                                    {
                                        ret = true;     // 上位のPtlPageかPtlEditFormXObjectでsetContentが必要
                                    }
                                }
                            }
                            break;
                        case TYPE_IMAGE:
                            {
                                PtlEditImage elemImage = (PtlEditImage)elem;
                                if (imageParh.substring(imageParh.length()-3).equals("pdf"))
                                {
                                    // FormXObjectを挿入して画像を削除する
                                    if (replaceImageWIthPdf(elems, i, elemImage, imageParh))
                                    {
                                        ret = true;     // 上位のPtlPageかPtlEditFormXObjectでsetContentが必要
                                    }
                                }
                                else
                                {
                                    // PtlEditImageの画像を直接置き換える
                                    if (replaceImageWithImage(elemImage, imageParh))
                                    {
                                        ret = true;     // 上位のPtlPageかPtlEditFormXObjectでsetContentが必要
                                    }
                                }
                            }
                            break;
                        default:
                            break;
                        }
                    }
                }
            }
        }
        catch (PtlException pex) {
            System.out.println("PtlException : ErrorCode = " + pex.getErrorCode() + "\n  " + pex.getErrorMessage());
        }
        catch (Exception ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        catch (Error ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        return ret;
    }

    // PtlEditImageの画像を直接置き換える
    static boolean replaceImageWithImage(PtlEditImage elemImage, String imageParh)
    {
        try
        {
            try (PtlParamInput input = new PtlParamInput(imageParh);
                 PtlParamEditImage paramEditImage = new PtlParamEditImage())
            {
                paramEditImage.setImageStream(input);
                paramEditImage.setUseOriginalJpegQuality(true);
                elemImage.setImage(paramEditImage); // PtlParamEditImageに設定した画像で置き換え
            }
            return true;    // Elementが所属するPtlPage若しくはPtlEditFormXObjectでsetContentが必要なのでtrueを返す
        }
        catch (PtlException pex) {
            System.out.println("PtlException : ErrorCode = " + pex.getErrorCode() + "\n  " + pex.getErrorMessage());
        }
        catch (Exception ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        catch (Error ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        // 何か問題が起こった
        return false;
    }

    // 新しいPtlEditFormXObjectにページを設定したものを挿入し、元のPtlEditImageは削除する
    static boolean replaceImageWIthPdf(PtlEditElements elems, int index, PtlEditImage elemImage, String pdfPath)
    {
        try
        {
            try (PtlParamInput input = new PtlParamInput(pdfPath);
                 PtlPDFDocument doc = new PtlPDFDocument())
            {
                doc.load(input);

                try (PtlPages pages = doc.getPages();
                     PtlPage page = pages.get(0);             // 1ページ目を使用する
                     PtlRect viewRect = page.getViewBox())    // ページサイズ
                {
                    float widthPage = viewRect.getRight() - viewRect.getLeft();     // ページ幅
                    float heightPage = viewRect.getTop() - viewRect.getBottom();    // ページ高さ
                    try (PtlEditFormXObject form = new PtlEditFormXObject(page);    // ページを元にPtlEditFormXObjectを作成
                         PtlTransMatrix matImage = elemImage.getMatrix();           // ページ若しくはFormXObject内での実Matrixを取得
                         PtlTransMatrix matRate = new PtlTransMatrix(1 / widthPage, 0, 0, 1 / heightPage, 0, 0);  // 元の画像が表示されていたサイズに合わせるためのMatrix
                         PtlTransMatrix matForm = matImage.concat(matRate))         // 元イメージのMatrixとページサイズよりFormXObjectのMatrixを決定
                    {
                        if (elemImage.hasClip())
                        {
                            try (PtlEditClip clip = elemImage.getClip();          // PtlEditImageが持っているクリッピングを取得
                                 PtlEditClip clipNew = clip.cloneCopy())
                            {
                                form.setClip(clipNew);                  // 元のElementが持っていたクリッピングを設定(setMatrixより先に行うこと)
                            }
                        }
                        form.setMatrix(matForm);
                        elems.insert(index, form);              // 新しいPtlEditImageを挿入
                        elems.remove(index + 1);                // 元のElementを削除
                    }
                }
            }
            return true;    // Elementが所属するPtlPage若しくはPtlEditFormXObjectでsetContentが必要なのでtrueを返す
        }
        catch (PtlException pex) {
            System.out.println("PtlException : ErrorCode = " + pex.getErrorCode() + "\n  " + pex.getErrorMessage());
        }
        catch (Exception ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        catch (Error ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
        // 何か問題が起こった
        return false;
    }
}
